﻿/**
* @FileName:     Transpose.cs
* @Programmer	 Brian Hinds
* @Date:         20-02-2017
* @Version:      1.0.2
*/

using System.Text.RegularExpressions;

namespace CCSPortWizard
{
    internal class Transposer
    {
        private const int HEADER_MAX = 100;
        private const int SYNTAX_MAX = 100;
        private string[,] SyntaxTable;
        private string CSNameSpace = "";

        public Transposer()
        {
        }

        /// <summary>
        /// This method manages the name space for the cs file
        /// </summary>
        /// <method name= "Transposer"></method>
        /// <param name="string CSNameSpace"></param>
        /// <returns>None</returns>
        public Transposer(string CSNameSpace)
        {
            this.CSNameSpace = CSNameSpace;

            SyntaxTable = new string[Transposer.HEADER_MAX, Transposer.SYNTAX_MAX];
            SyntaxTable[0, 0] = "generic";
            SyntaxTable[0, 1] = "main";
            SyntaxTable[0, 2] = "#include";
            SyntaxTable[0, 3] = "int";
            SyntaxTable[0, 4] = "float";
            SyntaxTable[0, 5] = "double";
            SyntaxTable[0, 6] = "char";
            SyntaxTable[0, 7] = "void";
            SyntaxTable[0, 8] = "NULL";
        }

        /// <summary>
        /// This method manages the syntax table indexes
        /// </summary>
        /// <method name= "SyntaxTableLastIndex"></method>
        /// <param name="int IndexA, int IndexB"></param>
        /// <returns>int</returns>
        private int SyntaxTableLastIndex(int IndexA, int IndexB)
        {
            int i = 0;

            switch (IndexA)
            {
                /// <summary>
                /// This case locates the first dimension in the syntax table index after the last first dimension index
                /// </summary>
                case 1:
                    while (SyntaxTable[i, 0] != null)
                    {
                        ++i;
                        if (i >= Transposer.HEADER_MAX) return -1;
                    }
                    break;

                /// <summary>
                /// This case locates the second dimension in the syntax table index after the last second dimension index
                /// </summary>
                case 2:
                    while (SyntaxTable[IndexB, i] != null)
                    {
                        ++i;
                        if (i >= Transposer.SYNTAX_MAX) return -1;
                    }
                    break;
            }
            return i - 1;
        }

        /// <summary>
        /// This method Manages the generic syntax transpose
        /// </summary>
        /// <method name= "Transpose_Generic_Syntax"></method>
        /// <param name="char Mode, ref string codeLine, int SyntaxMatchNumber"></param>
        /// <returns>void</returns>
        private void Transpose_Generic_Syntax(char Mode, ref string codeLine, int SyntaxMatchNumber)
        {
            switch (Mode)
            {
                /// <summary>
                /// keyword r triggers registry word
                /// </summary>
                case 'r':

                    int fileIndex = SyntaxTableLastIndex(1, -1) + 1;

                    SyntaxTable[fileIndex, 0] = "generic";
                    SyntaxTable[fileIndex, 1] = "main";
                    SyntaxTable[fileIndex, 2] = "#include";
                    SyntaxTable[fileIndex, 3] = "int";
                    SyntaxTable[fileIndex, 4] = "float";
                    SyntaxTable[fileIndex, 5] = "double";
                    SyntaxTable[fileIndex, 6] = "char";
                    SyntaxTable[fileIndex, 7] = "void";
                    SyntaxTable[fileIndex, 8] = "NULL";
                    break;

                /// <summary>
                /// Keyword c triggers the c-sharp code transpose
                /// </summary>
                case 'c':

                    // check which keyword is found in a line
                    switch (SyntaxMatchNumber)
                    {
                        /// <summary>
                        /// Checking for a main in the codes
                        /// </summary>
                        case 1:

                            string SyntaxPattern = @"(^|[^\w])([a-zA-Z]+)(\s+)(main)(.*)";
                            string Syntax_Main = "";
                            if (Regex.IsMatch(codeLine, SyntaxPattern))
                            {
                                Syntax_Main = Regex.Replace(codeLine, SyntaxPattern, main => "\t\tstatic " + main.Groups[1].Value +
                                                     main.Groups[2].Value + " Main(string[] args)");
                            }

                            codeLine = "namespace " + CSNameSpace + System.Environment.NewLine;
                            codeLine += "{" + System.Environment.NewLine;
                            codeLine += "\tclass Program" + System.Environment.NewLine;
                            codeLine += "\t{" + System.Environment.NewLine;
                            codeLine += Syntax_Main;

                            break;

                        /// <summary>
                        /// Checking for the #include tag in the codes
                        /// </summary>
                        case 2:

                            Match match = Regex.Match(codeLine, @"(^|[^\w])<stdio.h>([^\w]|$)");
                            if (match.Success)
                            {
                                Transpose_Studio_Syntax('r', ref codeLine, 0);

                                codeLine = @"using System;" + System.Environment.NewLine;
                                codeLine += @"using System.Collections.Generic;" + System.Environment.NewLine;
                                codeLine += @"using System.Text;";
                            }

                            /// <summary>
                            ///  checks keyword and delete code line
                            /// </summary>

                            match = Regex.Match(codeLine, @"(^|[^\w])<stdlib.h>([^\w]|$)");
                            if (match.Success)
                            {
                                Transpose_Stdlib_Syntax('r', ref codeLine, 0);
                                codeLine = "";
                            }

                            /// <summary>
                            /// Skips the string.h #include if found
                            /// </summary>

                            match = Regex.Match(codeLine, @"(^|[^\w])<string.h>([^\w]|$)");
                            if (match.Success)
                            {
                                Transpose_String_Syntax('r', ref codeLine, 0);
                                codeLine = "";
                            }

                            break;

                        /// <summary>
                        /// Checking for char
                        /// </summary>
                        case 6:

                            string patternCharArray = @"(^|[^\w])(char)(\s+)(\w+)(\[\d+\])";
                            if (Regex.IsMatch(codeLine, patternCharArray))
                            {
                                codeLine = Regex.Replace(codeLine, patternCharArray, main => main.Groups[1].Value + "string" + main.Groups[3].Value + main.Groups[4].Value);
                            }
                            break;

                        /// <summary>
                        /// Checking for NULL
                        /// </summary>
                        case 8:

                            string patternNull = @"(^|[^\w])(NULL)([^\w]|$)";
                            if (Regex.IsMatch(codeLine, patternNull))
                            {
                                codeLine = Regex.Replace(codeLine, patternNull, main => main.Groups[1].Value + "null" + main.Groups[3].Value);
                            }
                            break;
                    }

                    break;
            }
        }

        /// <summary>
        /// This method manages the stdio.h library transpose
        /// </summary>
        /// <method name= "Transpose_Studio_Syntax"></method>
        /// <param name="char Mode, ref string codeLine, int SyntaxMatchNumber"></param>
        /// <returns>void</returns>
        private void Transpose_Studio_Syntax(char Mode, ref string codeLine, int SyntaxMatchNumber)
        {
            switch (Mode)
            {
                /// <summary>
                /// The r triggers the registry words
                /// </summary>
                case 'r':

                    int fileIndex = SyntaxTableLastIndex(1, -1) + 1;

                    this.SyntaxTable[fileIndex, 0] = "stdio.h";
                    this.SyntaxTable[fileIndex, 1] = "printf";
                    this.SyntaxTable[fileIndex, 2] = "gets";
                    this.SyntaxTable[fileIndex, 3] = "FILE";
                    this.SyntaxTable[fileIndex, 4] = "fopen";
                    this.SyntaxTable[fileIndex, 5] = "fgets";
                    this.SyntaxTable[fileIndex, 6] = "fclose";
                    this.SyntaxTable[fileIndex, 7] = "fprintf";
                    break;

                /// <summary>
                /// The c triggers the transposing of c to c-sharp
                /// </summary>
                case 'c':

                    switch (SyntaxMatchNumber)
                    {
                        /// <summary>
                        /// This case triggers for printf syntax
                        /// </summary>
                        case 1:

                            string patternPrintf = @"(^|[^\w])(printf)([^\w])";
                            Regex rNewLine = new Regex(@"(\\n)(\s*"")");

                            /// <summary>
                            /// This condition includes a new line in the transpose
                            /// </summary>
                            if (rNewLine.IsMatch(codeLine))
                            {
                                codeLine = Regex.Replace(codeLine, patternPrintf, m => m.Groups[1].Value + @"System.Console.WriteLine" + m.Groups[3].Value);
                                codeLine = rNewLine.Replace(codeLine, m => "" + m.Groups[2].Value);
                            }
                            /// <summary>
                            /// This condition do not includes a new line in the transpose
                            /// </summary>
                            else
                            {
                                codeLine = Regex.Replace(codeLine, patternPrintf, m => m.Groups[1].Value + @"System.Console.Write" + m.Groups[3].Value);
                            }

                            string patternFormat = @"(""[^""^%]*)(%(\d)*([dDfFsSuUcC]))";
                            Match matchFormat = Regex.Match(codeLine, patternFormat);

                            int i = 0;
                            while (matchFormat.Success)
                            {
                                if (matchFormat.Groups[3].Success)
                                    codeLine = Regex.Replace(codeLine, patternFormat, m => m.Groups[1].Value + "{" + i++ +
                                    "," + matchFormat.Groups[3].Value + ":" + matchFormat.Groups[4].Value + "}");
                                else
                                    codeLine = Regex.Replace(codeLine, patternFormat, m => m.Groups[1].Value + "{" + i++ +
                                    ":" + matchFormat.Groups[4].Value + "}");
                                matchFormat = Regex.Match(codeLine, patternFormat);
                            }

                            break;

                        /// <summary>
                        /// This case searches for the gets syntax
                        /// </summary>
                        case 2:

                            string patternGets = @"(^|[^\w])(gets)(\s*)(\()(\w+)(\))";
                            if (Regex.IsMatch(codeLine, patternGets))
                            {
                                codeLine = Regex.Replace(codeLine, patternGets, m => m.Groups[1].Value + m.Groups[5].Value + " = " +
                                        @"System.Console.ReadLine()");
                            }

                            break;

                        /// <summary>
                        /// This case searches for the FILE syntax
                        /// </summary>
                        case 3:

                            string patternFopen = @"(^|[^\w])(fopen)([^\w])";
                            if (Regex.IsMatch(codeLine, patternFopen))
                            {
                                string patternFile = @"(^|[^\w])(FILE)([^\w])";
                                codeLine = Regex.Replace(codeLine, patternFile, m => "");
                            }
                            else
                            {
                                codeLine = "";
                            }

                            break;

                        /// <summary>
                        /// This case searches for the FOPEN syntax
                        /// </summary>
                        case 4:

                            string patternFopen1 = @"(^|[^\w])(\w+)(\s*=\s*)(fopen)(\s*\([^,]*)(,\s*"")([a-zA-Z]+)(""\s*\))";
                            Match match = Regex.Match(codeLine, patternFopen1);
                            if (match.Success)
                            {
                                /// <summary>
                                /// This condition is for write file
                                /// </summary>
                                if (match.Groups[7].Value == "w")
                                {
                                    codeLine = Regex.Replace(codeLine, patternFopen1, m => m.Groups[1].Value + "System.IO.StreamWriter " + m.Groups[2].Value +
                                             m.Groups[3].Value + "new System.IO.StreamWriter" + m.Groups[5].Value + ")");
                                }
                                /// <summary>
                                /// This condition is for read file
                                /// </summary>
                                else if (match.Groups[7].Value == "r")
                                {
                                    codeLine = Regex.Replace(codeLine, patternFopen1, m => m.Groups[1].Value + "System.IO.StreamReader " + m.Groups[2].Value +
                                             m.Groups[3].Value + "new System.IO.StreamReader" + m.Groups[5].Value + ")");
                                }
                            }

                            break;

                        /// <summary>
                        /// This case searches for fgets syntax
                        /// </summary>
                        case 5:

                            string patternFgets = @"(^|[^\w])(fgets)(\s*\()(\w+)(\s*,\s*[^,]+,\s*)(\w+)(\s*\)\s*)((!=)?(==)?)";
                            Match matchFgets = Regex.Match(codeLine, patternFgets);
                            if (matchFgets.Success)
                            {
                                if (matchFgets.Groups[8].Success)
                                {
                                    codeLine = Regex.Replace(codeLine, patternFgets, m => "(" + m.Groups[1].Value + m.Groups[4].Value +
                                            " = " + m.Groups[6].Value + ".ReadLine()" + ") " + m.Groups[8].Value);
                                }
                                else
                                {
                                    codeLine = Regex.Replace(codeLine, patternFgets, m => m.Groups[1].Value + m.Groups[4].Value +
                                            " = " + m.Groups[6].Value + ".ReadLine()");
                                }
                            }

                            break;

                        /// <summary>
                        /// This case searches for fclose syntax
                        /// </summary>
                        case 6: // keyword: fclose

                            string patternFclose = @"(^|[^\w])(fclose)(\s*\()(\w+)(\s*\))";
                            if (Regex.IsMatch(codeLine, patternFclose))
                            {
                                codeLine = Regex.Replace(codeLine, patternFclose, m => m.Groups[1].Value + m.Groups[4].Value +
                                                     ".Close()");
                            }

                            break;

                        /// <summary>
                        /// This case searches for fprintf syntax
                        /// </summary>
                        case 7:

                            string patternFprintf = @"(^|[^\w])(fprintf)(\s*\()(\w+)(,\s*)([^\)]*)";
                            Regex rNewLine1 = new Regex(@"(\\n)(\s*"")");

                            if (rNewLine1.IsMatch(codeLine))
                            {
                                codeLine = Regex.Replace(codeLine, patternFprintf, m => m.Groups[1].Value + m.Groups[4].Value +
                                             ".WriteLine(" + m.Groups[6].Value);
                                codeLine = rNewLine1.Replace(codeLine, m => "" + m.Groups[2].Value);
                            }
                            else
                            {
                                codeLine = Regex.Replace(codeLine, patternFprintf, m => m.Groups[1].Value + m.Groups[4].Value +
                                             ".Write(" + m.Groups[6].Value + ")");
                            }

                            break;
                    }

                    break;
            }
        }

        /// <summary>
        /// This method manages the stdlib.h library transpose
        /// </summary>
        /// <method name= "Transpose_Stdlib_Syntax"></method>
        /// <param name="char Mode, ref string codeLine, int SyntaxMatchNumber"></param>
        /// <returns>void</returns>
        private void Transpose_Stdlib_Syntax(char Mode, ref string codeLine, int SyntaxMatchNumber)
        {
            switch (Mode)
            {
                /// <summary>
                /// The r triggers the registry words
                /// </summary>
                case 'r':

                    int fileIndex = SyntaxTableLastIndex(1, -1) + 1;

                    this.SyntaxTable[fileIndex, 0] = "stdlib.h";
                    this.SyntaxTable[fileIndex, 1] = "atoi";
                    break;

                /// <summary>
                /// The c triggers the transposing of c to c-sharp
                /// </summary>
                case 'c':

                    switch (SyntaxMatchNumber)
                    {
                        /// <summary>
                        /// This case handles the transpose of atoi
                        /// </summary>
                        case 1:

                            string patternAtoi = @"(^|[^\w])(atoi)([^\w])";

                            if (Regex.IsMatch(codeLine, patternAtoi))
                            {
                                codeLine = Regex.Replace(codeLine, patternAtoi, m => m.Groups[1].Value + "Int32.Parse" + m.Groups[3].Value);
                            }

                            break;
                    }

                    break;
            }
        }

        /// <summary>
        /// This method manages the String.h library transpose
        /// </summary>
        /// <method name= "Transpose_String_Syntax"></method>
        /// <param name="char Mode, ref string codeLine, int SyntaxMatchNumber"></param>
        /// <returns>void</returns>
        private void Transpose_String_Syntax(char Mode, ref string codeLine, int SyntaxMatchNumber)
        {
            switch (Mode)
            {
                /// <summary>
                /// The r triggers the registry words
                /// </summary>
                case 'r':

                    int fileIndex = this.SyntaxTableLastIndex(1, -1) + 1;

                    SyntaxTable[fileIndex, 0] = "string.h";
                    SyntaxTable[fileIndex, 1] = "strlen";
                    break;

                /// <summary>
                /// The c triggers the transposing of c to c-sharp
                /// </summary>
                case 'c':

                    switch (SyntaxMatchNumber)
                    {
                        /// <summary>
                        /// This case handles the transpose from strlen
                        /// </summary>
                        case 1:

                            string SyntaxPattern = @"(^|[^\w])(strlen\s*\(\s*)([^)|^\s]+)(\s*\))";

                            if (Regex.IsMatch(codeLine, SyntaxPattern))
                            {
                                codeLine = Regex.Replace(codeLine, SyntaxPattern, m => m.Groups[1].Value + m.Groups[3].Value + ".Length");
                            }

                            break;
                    }

                    break;
            }
        }

        /// <summary>
        /// This method scan each line of object c code and transpose it to c-sharp
        /// </summary>
        /// <method name= "SyntaxScan"></method>
        /// <param name="ref string codeLine"></param>
        /// <returns>bool</returns>
        public bool SyntaxScan(ref string codeLine)
        {
            /// <summary>
            ///
            ///
            /// </summary>
            bool SyntaxMainScan = false;
            int IndexMaxA = SyntaxTableLastIndex(1, -1) + 1;

            /// <summary>
            /// This loop cycles through the header files syntax for a match
            /// </summary>
            for (int IndexHeader = 0; IndexHeader < IndexMaxA; ++IndexHeader)
            {
                /// <summary>
                /// This loops until a match is found
                /// </summary>
                int IndexMaxB = SyntaxTableLastIndex(2, IndexHeader) + 1;
                for (int IndexSyntax = 1; IndexSyntax < IndexMaxB; ++IndexSyntax)
                {
                    /// <summary>
                    /// While in looping process if a match is found then the transpose method is called
                    /// </summary>
                    Match SYNTAX = Regex.Match(codeLine, @"(^|[^\w])(" + SyntaxTable[IndexHeader, IndexSyntax] + @")([^\w]|$)");
                    if (SYNTAX.Success)
                    {
                        if (SyntaxTable[IndexHeader, 0] == "generic") { Transpose_Generic_Syntax('c', ref codeLine, IndexSyntax); }
                        if (SyntaxTable[IndexHeader, 0] == "stdio.h") { Transpose_Studio_Syntax('c', ref codeLine, IndexSyntax); }
                        if (SyntaxTable[IndexHeader, 0] == "stdlib.h") { Transpose_Stdlib_Syntax('c', ref codeLine, IndexSyntax); }
                        if (SyntaxTable[IndexHeader, 0] == "string.h") { Transpose_String_Syntax('c', ref codeLine, IndexSyntax); }
                        if (SyntaxTable[IndexHeader, IndexSyntax] == "main") { SyntaxMainScan = true; }
                    }
                }
            }

            return SyntaxMainScan;
        }
    }
}